/**
 * @geometryzen/newton-jsxgraph-widgets 0.9.1
 * (c) David Geo Holmes
 * Released under the MIT License.
 */
import { Geometric2, Unit } from '@geometryzen/multivectors';
import { COORDS_BY_USER, Board, JSXGraph } from 'jsxgraph';
import { Particle2, Rod2 } from '@geometryzen/newton';

/**
 * Sets the postion of the geometry element.
 * For the change to take effect, the board must be updated.
 * @param element The geometry element to be positioned.
 * @param position The desired position vector.
 * @returns A reference to the geometry element.
 */
function setPosition(element, position) {
    return element.setPosition(COORDS_BY_USER, [position.x, position.y]);
}
/*
const coords: [number, number] = [0, 0];

export function setPositionII(point: Point, position: Geometric2) {
    coords[0] = position.x;
    coords[1] = position.y;
    point.setPosition(COORDS_BY_USER, coords);
}
*/

// Copyright 2021-2022 David Geo Holmes.  All Rights Reserved.
const x$1 = new Geometric2();
class JsxAngle {
    board;
    angle;
    $center = Geometric2.vector(0, 0);
    $rotor = Geometric2.rotorFromDirections(Geometric2.e1, Geometric2.e2);
    radiusPoint;
    centerPoint;
    rotatePoint;
    $base = Geometric2.vector(1, 0);
    $visible = true;
    constructor(board, attributes) {
        this.board = board;
        this.radiusPoint = board.create('point', [1, 0], { visible: false });
        this.centerPoint = board.create('point', [0, 0], { visible: false });
        this.rotatePoint = board.create('point', [0, 1], { visible: false });
        this.angle = board.create('angle', [this.radiusPoint, this.centerPoint, this.rotatePoint], attributes);
    }
    get base() {
        return this.$base;
    }
    set base(base) {
        this.$base.copyVector(base).direction();
    }
    get center() {
        return this.$center;
    }
    set center(pos) {
        this.$center.copyVector(pos);
    }
    get rotor() {
        return this.$rotor;
    }
    set rotor(value) {
        this.$rotor.copySpinor(value);
    }
    /**
     * @override
     */
    removeFromBoard() {
        this.board.suspendUpdate();
        try {
            this.board.removeObject(this.angle, false);
            this.board.removeObject(this.rotatePoint, false);
            this.board.removeObject(this.centerPoint, false);
            this.board.removeObject(this.radiusPoint, false);
        }
        finally {
            this.board.unsuspendUpdate();
        }
    }
    /**
     * @override
     */
    update() {
        if (this.$visible) {
            x$1.copyVector(this.$center);
            setPosition(this.centerPoint, x$1);
            x$1.copyVector(this.$base).addVector(this.$center);
            setPosition(this.radiusPoint, x$1);
            x$1.copyVector(this.$base).rotate(this.$rotor).addVector(this.$center);
            // angle.radiusPoint and angle.anglePoint (from Sector) are undefined.
            setPosition(this.rotatePoint, x$1);
        }
    }
    get visible() {
        return this.$visible;
    }
    set visible(visible) {
        if (visible !== this.$visible) {
            this.angle.setAttribute({ visible });
            this.$visible = visible;
        }
    }
}

// Copyright 2021-2022 David Geo Holmes.  All Rights Reserved.
const position = new Geometric2();
class JsxArrow {
    board;
    arrow;
    head;
    tail;
    $vector = Geometric2.vector(1, 0);
    $pos = Geometric2.vector(0, 0);
    $visible = true;
    constructor(board, attributes) {
        this.board = board;
        this.arrow = board.create('arrow', [[0, 0], [0, 0]], attributes);
        if (attributes && attributes.mutableHead) {
            const color = attributes ? attributes.color : void 0;
            const head = board.create('point', [1, 0], { withLabel: false, color });
            this.head = head;
            this.updateHead();
            head.on('down', (e) => {
                e.preventDefault();
            });
            head.on('drag', (e) => {
                e.preventDefault();
                this.$vector.x = head.X() - this.$pos.x;
                this.$vector.y = head.Y() - this.$pos.y;
            });
            head.on('up', (e) => {
                e.preventDefault();
                this.$vector.x = head.X() - this.$pos.x;
                this.$vector.y = head.Y() - this.$pos.y;
            });
        }
        else {
            this.head = void 0;
        }
        if (attributes && attributes.mutableTail) {
            const color = attributes ? attributes.color : void 0;
            const tail = board.create('point', [0, 0], { withLabel: false, color });
            this.tail = tail;
            this.updateTail();
            tail.on('down', (e) => {
                e.preventDefault();
            });
            tail.on('drag', (e) => {
                e.preventDefault();
                const px = this.$pos.x;
                const py = this.$pos.y;
                this.$pos.x = tail.X();
                this.$pos.y = tail.Y();
                this.$vector.x = this.$vector.x + px - this.$pos.x;
                this.$vector.y = this.$vector.y + py - this.$pos.y;
            });
            tail.on('up', (e) => {
                e.preventDefault();
                const px = this.$pos.x;
                const py = this.$pos.y;
                this.$pos.x = tail.X();
                this.$pos.y = tail.Y();
                this.$vector.x = this.$vector.x + px - this.$pos.x;
                this.$vector.y = this.$vector.y + py - this.$pos.y;
            });
        }
        else {
            this.tail = void 0;
        }
    }
    get pos() {
        return this.$pos;
    }
    set pos(pos) {
        this.$pos.copyVector(pos);
        if (this.tail) {
            this.updateTail();
        }
        if (this.head) {
            this.updateHead();
        }
    }
    get vector() {
        return this.$vector;
    }
    set vector(value) {
        this.$vector.copyVector(value);
        if (this.head) {
            this.updateHead();
        }
    }
    removeFromBoard() {
        const board = this.board;
        board.suspendUpdate();
        try {
            if (this.tail) {
                this.board.removeObject(this.tail, false);
            }
            if (this.head) {
                this.board.removeObject(this.head, false);
            }
            this.board.removeObject(this.arrow, false);
        }
        finally {
            board.unsuspendUpdate();
        }
    }
    update() {
        position.copyVector(this.$pos);
        setPosition(this.arrow.point1, position);
        position.copyVector(this.$pos).addVector(this.$vector);
        setPosition(this.arrow.point2, position);
    }
    get visible() {
        return this.$visible;
    }
    set visible(visible) {
        if (visible !== this.$visible) {
            if (visible) {
                this.arrow.setAttribute({ 'visible': true });
            }
            else {
                this.arrow.setAttribute({ 'visible': false });
            }
            this.$visible = visible;
        }
    }
    updateTail() {
        if (this.tail) {
            position.copyVector(this.$pos);
            setPosition(this.tail, position);
        }
    }
    updateHead() {
        if (this.head) {
            position.copyVector(this.$pos).addVector(this.$vector);
            setPosition(this.head, position);
        }
    }
}

// Copyright 2021-2022 David Geo Holmes.  All Rights Reserved.
const e1$3 = Geometric2.e1;
const e2$2 = Geometric2.e2;
const scratch$1 = new Geometric2([0, 0, 0, 0]);
const scratchW = new Geometric2([0, 0, 0, 0]);
const scratchH = new Geometric2([0, 0, 0, 0]);
class JsxBlock {
    board;
    block;
    p1;
    p2;
    p3;
    p4;
    polygon;
    $scaleL = Geometric2.scalar(1);
    $visible = true;
    constructor(board, block, attributes) {
        this.board = board;
        this.block = block;
        this.p1 = board.create('point', [0.0, 0.0], { visible: false });
        this.p2 = board.create('point', [0.0, 0.0], { visible: false });
        this.p3 = board.create('point', [0.0, 0.0], { visible: false });
        this.p4 = board.create('point', [0.0, 0.0], { visible: false });
        this.polygon = board.create('polygon', [this.p1, this.p2, this.p3, this.p4], attributes);
        block.uuid = this.polygon.id;
    }
    get scaleL() {
        return this.$scaleL;
    }
    set scaleL(scaleL) {
        this.$scaleL.copyScalar(scaleL.a, scaleL.uom);
    }
    removeFromBoard() {
        const board = this.board;
        board.suspendUpdate();
        try {
            this.board.removeObject(this.polygon, false);
            this.board.removeObject(this.p4, false);
            this.board.removeObject(this.p3, false);
            this.board.removeObject(this.p2, false);
            this.board.removeObject(this.p1, false);
        }
        finally {
            board.unsuspendUpdate();
        }
    }
    update() {
        if (this.$visible) {
            const block = this.block;
            const X = block.X;
            const R = block.R;
            const scaleL = this.scaleL;
            scratchW.copy(e1$3).rotate(R).mul(block.width);
            scratchH.copy(e2$2).rotate(R).mul(block.height);
            scratch$1.copy(scratchW).add(scratchH).mulByNumber(0.5).neg().add(X).divByScalar(scaleL.a, scaleL.uom);
            setPosition(this.p1, scratch$1);
            scratch$1.copy(scratchW).sub(scratchH).mulByNumber(0.5).add(X).divByScalar(scaleL.a, scaleL.uom);
            setPosition(this.p2, scratch$1);
            scratch$1.copy(scratchW).add(scratchH).mulByNumber(0.5).add(X).divByScalar(scaleL.a, scaleL.uom);
            setPosition(this.p3, scratch$1);
            scratch$1.copy(scratchW).sub(scratchH).mulByNumber(0.5).neg().add(X).divByScalar(scaleL.a, scaleL.uom);
            setPosition(this.p4, scratch$1);
        }
    }
    get visible() {
        return this.$visible;
    }
    set visible(visible) {
        if (visible !== this.$visible) {
            this.polygon.setAttribute({ visible });
            this.$visible = visible;
        }
    }
}

// Copyright 2021-2022 David Geo Holmes.  All Rights Reserved.
const one$1 = Geometric2.one;
const m$1 = Geometric2.one;
const N$1 = Geometric2.one;
const scratchX$2 = new Geometric2([0, 0, 0, 0]);
const scratchF$1 = new Geometric2([0, 0, 0, 0]);
class JsxConstraint {
    forceLaw;
    scaleL = one$1.divByScalar(m$1.a, m$1.uom);
    scaleF = one$1.divByScalar(N$1.a, N$1.uom);
    visible;
    arrows = [];
    constructor(board, forceLaw, options) {
        this.forceLaw = forceLaw;
        for (let i = 0; i < 1; i++) {
            const atts = {
                strokeWidth: 2,
                lastArrow: { type: 1 }
            };
            if (options) {
                if (typeof options.name === 'string') {
                    atts.withLabel = true;
                    atts.name = options.name;
                }
            }
            if (options) {
                if (typeof options.visible === 'boolean') {
                    atts.visible = options.visible;
                    this.visible = options.visible;
                }
            }
            this.arrows.push(board.create('arrow', [[0, 0], [0, 0]], atts));
        }
        this.visible = true;
    }
    removeFromBoard() {
        throw new Error('Method not implemented.');
    }
    update() {
        const X = this.forceLaw.getBody().X;
        const F = this.forceLaw.N;
        const scaleL = this.scaleL;
        const scaleF = this.scaleF;
        scratchX$2.copy(X).mul(scaleL);
        scratchF$1.copy(F).mul(scaleF);
        setPosition(this.arrows[0].point1, scratchX$2);
        scratchX$2.add(scratchF$1);
        setPosition(this.arrows[0].point2, scratchX$2);
        if (this.visible) ;
    }
}

// Copyright 2021-2022 David Geo Holmes.  All Rights Reserved.
const e1$2 = Geometric2.e1;
const scratch = new Geometric2([0, 0, 0, 0]);
class JsxDisc {
    board;
    disc;
    centerPoint;
    radiusPoint;
    circle;
    $scaleL = Geometric2.scalar(1);
    $visible = true;
    constructor(board, disc, attributes) {
        this.board = board;
        this.disc = disc;
        this.centerPoint = board.create('point', [0, 0], { color: 'red', visible: false });
        this.radiusPoint = board.create('point', [0, 1], { color: 'red', visible: false });
        this.circle = board.create('circle', [this.centerPoint, this.radiusPoint], attributes);
    }
    get scaleL() {
        return this.$scaleL;
    }
    set scaleL(scaleL) {
        this.$scaleL.copyScalar(scaleL.a, scaleL.uom);
    }
    removeFromBoard() {
        const board = this.board;
        board.suspendUpdate();
        try {
            this.board.removeObject(this.circle, false);
            this.board.removeObject(this.radiusPoint, false);
            this.board.removeObject(this.centerPoint, false);
        }
        finally {
            board.unsuspendUpdate();
        }
    }
    update() {
        if (this.$visible) {
            const disc = this.disc;
            const X = disc.X;
            const r = disc.radius;
            const scaleL = this.$scaleL;
            scratch.copy(X).divByScalar(scaleL.a, scaleL.uom);
            setPosition(this.centerPoint, scratch);
            scratch.copy(r).mul(e1$2).add(X).divByScalar(scaleL.a, scaleL.uom);
            setPosition(this.radiusPoint, scratch);
        }
    }
    get visible() {
        return this.$visible;
    }
    set visible(visible) {
        if (visible !== this.$visible) {
            this.circle.setAttribute({ visible });
            this.$visible = visible;
        }
    }
}

// Copyright 2021-2022 David Geo Holmes.  All Rights Reserved.
const diagramX = new Geometric2();
const diagramF = new Geometric2();
class JsxForce {
    board;
    force;
    arrow;
    $scaleF = Geometric2.scalar(1);
    $scaleL = Geometric2.scalar(1);
    $visible = true;
    constructor(board, force, attributes) {
        this.board = board;
        this.force = force;
        this.arrow = board.create('arrow', [[0, 0], [0, 0]], attributes);
    }
    removeFromBoard() {
        const board = this.board;
        board.suspendUpdate();
        try {
            this.board.removeObject(this.arrow, false);
        }
        finally {
            board.unsuspendUpdate();
        }
    }
    update() {
        if (this.$visible) {
            diagramX.copyVector(this.force.x);
            diagramX.divByScalar(this.scaleL.a, this.scaleL.uom);
            diagramF.copyVector(this.force.F);
            diagramF.divByScalar(this.scaleF.a, this.scaleF.uom);
            if (!Unit.isOne(diagramX.uom)) {
                throw new Error(`JsxForce.update() unable to show force.x ${this.force.x}. Change JsxForce.scaleX so that force.x divided by scaleX is dimensionless.`);
            }
            if (!Unit.isOne(diagramF.uom)) {
                throw new Error(`JsxForce.update() unable to show force.F ${this.force.F}. Change JsxForce.scaleF so that force.F divided by scaleF is dimensionless.`);
            }
            setPosition(this.arrow.point1, diagramX);
            diagramX.addVector(diagramF);
            setPosition(this.arrow.point2, diagramX);
        }
    }
    get scaleF() {
        return this.$scaleF;
    }
    set scaleF(scaleF) {
        this.$scaleF.copyScalar(scaleF.a, scaleF.uom);
    }
    get scaleL() {
        return this.$scaleL;
    }
    set scaleL(scaleL) {
        this.$scaleL.copyScalar(scaleL.a, scaleL.uom);
    }
    get visible() {
        // console.log(`attributes=>${JSON.stringify(this.arrow.getAttributes()}`))
        return this.$visible;
    }
    set visible(visible) {
        if (visible !== this.$visible) {
            if (visible) {
                this.arrow.showElement();
                // this.arrow.setDisplayRendNode(true)
            }
            else {
                this.arrow.hideElement();
                // this.arrow.setDisplayRendNode(false)
            }
            this.$visible = visible;
        }
    }
}

// Copyright 2021-2022 David Geo Holmes.  All Rights Reserved.
/**
 * A rendering of a davinci-newton Particle2 on a JsxGraph board.
 */
class JsxParticle {
    board;
    particle;
    point;
    $visible = true;
    /**
     * @param board The JsxGraph board that this JsxParticle will be rendered to.
     * @param particle The model for this rendering.
     * @param attributes The optional attributes for the creation of the underlying Point.
     */
    constructor(board, particle, attributes) {
        this.board = board;
        this.particle = particle;
        if (!(board instanceof Board)) {
            throw new Error("board must be a Board in \"new JsxParticle(board: Board, particle: Particle2, attributes?: PointAttributes)\".");
        }
        if (!(particle instanceof Particle2)) {
            throw new Error("particle must be a Particle2 in \"new JsxParticle(board: Board, particle: Particle2, attributes?: PointAttributes)\".");
        }
        this.point = board.create('point', [0, 0], attributes);
        this.update();
    }
    removeFromBoard() {
        const board = this.board;
        board.suspendUpdate();
        try {
            board.removeObject(this.point, false);
        }
        finally {
            board.unsuspendUpdate();
        }
    }
    update() {
        setPosition(this.point, this.particle.X);
    }
    get visible() {
        return this.$visible;
    }
    set visible(visible) {
        if (visible !== this.$visible) {
            this.point.setAttribute({ visible });
            this.$visible = visible;
        }
    }
}

// Copyright 2021-2022 David Geo Holmes.  All Rights Reserved.
/**
 * Working storage to avoid temporary object creation during update() method execution.
 */
const tempX$1 = new Geometric2();
const rotA = new Geometric2();
class JsxRod {
    board;
    rod;
    negPoint;
    posPoint;
    segment;
    $scaleL = Geometric2.scalar(1);
    $visible = true;
    constructor(board, rod, attributes) {
        this.board = board;
        this.rod = rod;
        if (!(board instanceof Board)) {
            throw new Error("board must be a Board in \"new JsxRod(board: Board, rod: Rod2, attributes?: SegmentAttributes)\".");
        }
        if (!(rod instanceof Rod2)) {
            throw new Error("rod must be a Rod2 in \"new JsxRod(board: Board, rod: Rod2, attributes?: SegmentAttributes)\".");
        }
        this.negPoint = board.create('point', [0, 0], { visible: false });
        this.posPoint = board.create('point', [0, 0], { visible: false });
        this.segment = board.create('segment', [this.negPoint, this.posPoint], attributes);
    }
    get scaleL() {
        return this.$scaleL;
    }
    set scaleL(scaleL) {
        this.$scaleL.copyScalar(scaleL.a, scaleL.uom);
    }
    removeFromBoard() {
        const board = this.board;
        board.suspendUpdate();
        try {
            this.board.removeObject(this.segment, false);
            this.board.removeObject(this.posPoint, false);
            this.board.removeObject(this.negPoint, false);
        }
        finally {
            board.unsuspendUpdate();
        }
    }
    update() {
        if (this.$visible) {
            const rod = this.rod;
            const X = rod.X;
            const R = rod.R;
            const a = rod.a;
            rotA.copy(a).rotate(R);
            tempX$1.copy(rotA).neg().add(X).divByScalar(this.scaleL.a, this.scaleL.uom);
            if (!Unit.isOne(tempX$1.uom)) {
                throw new Error(`scaleL =${this.$scaleL}, tempX=${tempX$1} JsxRod.update() unable to show rod.a (${this.rod.a}). Change JsxRod.scaleL so that rod.a divided by scaleL is dimensionless.`);
            }
            setPosition(this.negPoint, tempX$1);
            tempX$1.copy(rotA).add(X).divByScalar(this.scaleL.a, this.scaleL.uom);
            if (!Unit.isOne(tempX$1.uom)) {
                throw new Error(`scaleL =${this.$scaleL}, tempX=${tempX$1} JsxRod.update() unable to show rod.a (${this.rod.a}). Change JsxRod.scaleL so that rod.a divided by scaleL is dimensionless.`);
            }
            setPosition(this.posPoint, tempX$1);
        }
    }
    get visible() {
        return this.$visible;
    }
    set visible(visible) {
        if (visible !== this.$visible) {
            this.segment.setAttribute({ visible });
            this.$visible = visible;
        }
    }
}

// Copyright 2021-2022 David Geo Holmes.  All Rights Reserved.
const e1$1 = Geometric2.e1;
const e2$1 = Geometric2.e2;
const R$1 = new Geometric2().rotorFromDirections(e1$1, e2$1);
const d = new Geometric2();
const e = new Geometric2();
const L = new Geometric2();
const x = new Geometric2();
class JsxSpring {
    board;
    spring;
    p1;
    p2;
    points = [];
    segments = [];
    $visible = true;
    $scaleL = Geometric2.scalar(1);
    constructor(board, spring, config) {
        this.board = board;
        this.spring = spring;
        const radius = config.radius;
        const rings = config.rings;
        const quarters = 4 * rings;
        const p1 = board.create('point', [0, 0], { visible: false });
        this.points.push(p1);
        const p2 = board.create('point', [0, 0], { visible: false });
        const N = rings * 2;
        for (let i = 0; i < N; i++) {
            this.points.push(board.create('point', [
                ((k) => {
                    return () => {
                        d.x = p2.X() - p1.X();
                        d.y = p2.Y() - p1.Y();
                        L.copyVector(d).magnitude();
                        d.direction();
                        e.copyVector(d).rotate(R$1).direction();
                        const offset = radius * (1 - 2 * (k % 2)) * e.x;
                        const step = L.a / quarters;
                        return p1.X() + (2 * k + 1) * step * d.x + offset;
                    };
                })(i),
                ((k) => {
                    return () => {
                        d.x = p2.X() - p1.X();
                        d.y = p2.Y() - p1.Y();
                        L.copyVector(d).magnitude();
                        d.direction();
                        e.copyVector(d).rotate(R$1).direction();
                        const offset = radius * (1 - 2 * (k % 2)) * e.y;
                        const step = L.a / quarters;
                        return p1.Y() + (2 * k + 1) * step * d.y + offset;
                    };
                })(i)
            ], { visible: false }));
        }
        this.points.push(p2);
        const numSegments = this.points.length - 1;
        for (let i = 0; i < numSegments; i++) {
            this.segments.push(board.create('segment', [this.points[i], this.points[i + 1]], { color: 'black', strokeWidth: 1 }));
        }
        this.p1 = p1;
        this.p2 = p2;
    }
    get scaleL() {
        return this.$scaleL;
    }
    set scaleL(scaleL) {
        this.$scaleL.copyScalar(scaleL.a, scaleL.uom);
    }
    removeFromBoard() {
        const board = this.board;
        board.suspendUpdate();
        try {
            this.board.removeObject(this.segments, false);
            this.board.removeObject(this.points, false);
            this.board.removeObject(this.p2, false);
            this.board.removeObject(this.p1, false);
        }
        finally {
            board.unsuspendUpdate();
        }
    }
    update() {
        if (this.$visible) {
            const scaleL = this.scaleL;
            x.copyVector(this.spring.end1).divByScalar(scaleL.a, scaleL.uom);
            setPosition(this.p1, x);
            x.copyVector(this.spring.end2).divByScalar(scaleL.a, scaleL.uom);
            setPosition(this.p2, x);
        }
    }
    get visible() {
        return this.$visible;
    }
    set visible(visible) {
        if (visible !== this.$visible) {
            if (visible) {
                for (const segment of this.segments) {
                    segment.setAttribute({ 'visible': true });
                }
            }
            else {
                for (const segment of this.segments) {
                    segment.setAttribute({ 'visible': false });
                }
            }
            this.$visible = visible;
        }
    }
    get end1() {
        return this.p1;
    }
}

/**
 * A wrapper around a JSXGraph board providing methods for adding objects to
 * a Free Body Diagram.
 */
class JsxDiagram {
    board;
    views = [];
    /**
     * @param elementId HTML identifier (id) of the DIV element in which the diagram is rendered.
     * @param attributes an object that sets some of the diagram properties.
     */
    constructor(elementId, attributes) {
        this.board = JSXGraph.initBoard(elementId, attributes);
    }
    addAngle(attributes) {
        const view = new JsxAngle(this.board, attributes);
        this.views.push(view);
        return view;
    }
    addArrow(attributes) {
        const view = new JsxArrow(this.board, attributes);
        this.views.push(view);
        return view;
    }
    addDisc(disc, attributes) {
        const view = new JsxDisc(this.board, disc, attributes);
        this.views.push(view);
        return view;
    }
    addForce(force, attributes) {
        const view = new JsxForce(this.board, force, attributes);
        this.views.push(view);
        return view;
    }
    addParticle(particle, attributes) {
        const view = new JsxParticle(this.board, particle, attributes);
        this.views.push(view);
        return view;
    }
    addRod(rod, attributes) {
        const view = new JsxRod(this.board, rod, attributes);
        this.views.push(view);
        return view;
    }
    addSpring(spring, attributes) {
        const view = new JsxSpring(this.board, spring, attributes);
        this.views.push(view);
        return view;
    }
    removeAllFromBoard() {
        const board = this.board;
        board.suspendUpdate();
        try {
            const views = this.views;
            for (const view of views) {
                view.removeFromBoard();
            }
            views.length = 0;
        }
        finally {
            board.unsuspendUpdate();
        }
    }
    update() {
        const views = this.views;
        for (const view of views) {
            view.update();
        }
        this.board.update();
    }
}

// Copyright 2021-2022 David Geo Holmes.  All Rights Reserved.
const one = Geometric2.one;
const m = Geometric2.one;
const N = Geometric2.one;
const scratchX$1 = new Geometric2([0, 0, 0, 0]);
const scratchF = new Geometric2([0, 0, 0, 0]);
class JsxForceLaw {
    forceLaw;
    scaleL = one.clone().copyScalar(m.a, m.uom);
    scaleF = one.clone().copyScalar(N.a, N.uom);
    visible;
    arrows = [];
    constructor(board, forceLaw, options) {
        this.forceLaw = forceLaw;
        const forces = this.forceLaw.updateForces();
        this.visible = visibleFlags(forceLaw, options);
        for (let i = 0; i < forces.length; i++) {
            const atts = { strokeWidth: 2, lastArrow: { type: 1 }, visible: this.visible[i] };
            if (options && Array.isArray(options.names)) {
                atts.withLabel = true;
                atts.name = options.names[i];
            }
            if (options && Array.isArray(options.colors)) {
                atts.color = options.colors[i];
            }
            this.arrows.push(board.create('arrow', [[0, 0], [0, 0]], atts));
            this.visible.push(true);
        }
    }
    update() {
        const forces = this.forceLaw.updateForces();
        const scaleL = this.scaleL;
        const scaleF = this.scaleF;
        for (let i = 0; i < forces.length; i++) {
            scratchX$1.copy(forces[i].x).divByScalar(scaleL.a, scaleL.uom);
            scratchF.copy(forces[i].F).divByScalar(scaleF.a, scaleF.uom);
            setPosition(this.arrows[i].point1, scratchX$1);
            scratchX$1.add(scratchF);
            setPosition(this.arrows[i].point2, scratchX$1);
            this.arrows[i].setAttribute({ visible: this.visible[i] });
        }
    }
}
function visibleFlags(forceLaw, options) {
    if (options && Array.isArray(options.visible)) {
        return options.visible;
    }
    else {
        const flags = [];
        const forces = forceLaw.updateForces();
        // eslint-disable-next-line @typescript-eslint/no-unused-vars
        for (const _force of forces) {
            flags.push(true);
        }
        return flags;
    }
}

// Copyright 2021-2022 David Geo Holmes.  All Rights Reserved.
const tempX = Geometric2.scalar(0);
class JsxPolygon {
    board;
    body;
    jxgPoints = [];
    jxgX;
    polygon;
    $scaleL = Geometric2.scalar(1);
    $visible = true;
    constructor(board, body, attributes) {
        this.board = board;
        this.body = body;
        const N = body.rs.length;
        const jxgPoints = this.jxgPoints;
        for (let i = 0; i < N; i++) {
            jxgPoints.push(board.create('point', [0, 0], { visible: false }));
        }
        this.polygon = board.create('polygon', jxgPoints, attributes);
        this.jxgX = board.create('point', [0, 0], { visible: false });
    }
    get scaleL() {
        return this.$scaleL;
    }
    set scaleL(scaleL) {
        this.$scaleL.copyScalar(scaleL.a, scaleL.uom);
    }
    removeFromBoard() {
        const board = this.board;
        board.suspendUpdate();
        try {
            board.removeObject(this.jxgX, false);
            board.removeObject(this.polygon, false);
            board.removeObject(this.jxgPoints, false);
        }
        finally {
            board.unsuspendUpdate();
        }
    }
    update() {
        if (this.$visible) {
            const scaleL = this.scaleL;
            const X = this.body.X;
            const R = this.body.R;
            const jxgPoints = this.jxgPoints;
            const rs = this.body.rs;
            const N = jxgPoints.length;
            for (let i = 0; i < N; i++) {
                const jxgPoint = jxgPoints[i];
                const r = rs[i];
                tempX.copyVector(r).rotate(R).addVector(X).divByScalar(scaleL.a, scaleL.uom);
                setPosition(jxgPoint, tempX);
            }
            tempX.copyVector(X).divByScalar(scaleL.a, scaleL.uom);
            setPosition(this.jxgX, X);
        }
    }
    get visible() {
        return this.$visible;
    }
    set visible(visible) {
        if (visible !== this.$visible) {
            if (visible) {
                this.polygon.setAttribute({ 'visible': true });
            }
            else {
                this.polygon.setAttribute({ 'visible': false });
            }
            this.$visible = visible;
        }
    }
}

// Copyright 2021-2022 David Geo Holmes.  All Rights Reserved.
const e1 = Geometric2.e1;
const e2 = Geometric2.e2;
const R = Geometric2.rotorFromVectorToVector(e1, e2);
const scratchD = new Geometric2([0, 0, 0, 0]);
const scratchX = new Geometric2([0, 0, 0, 0]);
class JsxTapeMeasure {
    beginFn;
    endFn;
    b;
    e;
    scaleL = Geometric2.one;
    standoff = 0;
    tapemeasure;
    $visible = true;
    constructor(board, beginFn, endFn) {
        this.beginFn = beginFn;
        this.endFn = endFn;
        const b = board.create('point', [0.0, 0.0], { visible: false });
        const e = board.create('point', [0.0, 0.0], { visible: false });
        this.tapemeasure = board.create('tapemeasure', [
            [
                function () {
                    return b.X();
                },
                function () {
                    return b.Y();
                }
            ],
            [
                function () {
                    return e.X();
                },
                function () {
                    return e.Y();
                }
            ]
        ]);
        this.b = b;
        this.e = e;
    }
    removeFromBoard() {
        throw new Error('Method not implemented.');
    }
    update() {
        const b = this.beginFn();
        const e = this.endFn();
        const scaleL = this.scaleL;
        scratchD.copy(e).sub(b).direction().rotate(R).mulByNumber(this.standoff);
        scratchX.copy(b).divByScalar(scaleL.a, scaleL.uom).add(scratchD);
        setPosition(this.b, scratchX);
        scratchX.copy(e).divByScalar(scaleL.a, scaleL.uom).add(scratchD);
        setPosition(this.e, scratchX);
    }
    get visible() {
        return this.$visible;
    }
    set visible(visible) {
        if (visible !== this.$visible) {
            this.tapemeasure.setAttribute({ visible });
            this.$visible = visible;
        }
    }
}

export { JsxAngle, JsxArrow, JsxBlock, JsxConstraint, JsxDiagram, JsxDisc, JsxForce, JsxForceLaw, JsxParticle, JsxPolygon, JsxRod, JsxSpring, JsxTapeMeasure };
//# sourceMappingURL=index.js.map
