import { Geometric2 } from '@geometryzen/multivectors';
import { Point, Board, AngleAttributes, ArrowAttributes, PolygonAttributes, CircleAttributes, PointAttributes, SegmentAttributes, BoardAttributes } from 'jsxgraph';
import { Block2, SurfaceConstraint2, Disc2, Force2, Particle2, Rod2, Spring, Geometric2 as Geometric2$1, ForceLaw, Polygon2 } from '@geometryzen/newton';

interface JsxWidget {
    removeFromBoard(): void;
    update(): void;
}

declare class JsxAngle implements JsxWidget {
    private readonly board;
    private readonly angle;
    private readonly $center;
    private readonly $rotor;
    readonly radiusPoint: Point;
    readonly centerPoint: Point;
    readonly rotatePoint: Point;
    private readonly $base;
    private $visible;
    constructor(board: Board, attributes?: Partial<AngleAttributes>);
    get base(): Geometric2;
    set base(base: Geometric2);
    get center(): Geometric2;
    set center(pos: Geometric2);
    get rotor(): Geometric2;
    set rotor(value: Geometric2);
    /**
     * @override
     */
    removeFromBoard(): void;
    /**
     * @override
     */
    update(): void;
    get visible(): boolean;
    set visible(visible: boolean);
}

interface JsxArrowAttributes extends ArrowAttributes {
    mutableHead: boolean;
    mutableTail: boolean;
}
declare class JsxArrow implements JsxWidget {
    private readonly board;
    private readonly arrow;
    private readonly head;
    private readonly tail;
    private readonly $vector;
    private readonly $pos;
    private $visible;
    constructor(board: Board, attributes?: Partial<JsxArrowAttributes>);
    get pos(): Geometric2;
    set pos(pos: Geometric2);
    get vector(): Geometric2;
    set vector(value: Geometric2);
    removeFromBoard(): void;
    update(): void;
    get visible(): boolean;
    set visible(visible: boolean);
    private updateTail;
    private updateHead;
}

declare class JsxBlock implements JsxWidget {
    private readonly board;
    private readonly block;
    private readonly p1;
    private readonly p2;
    private readonly p3;
    private readonly p4;
    private readonly polygon;
    private readonly $scaleL;
    private $visible;
    constructor(board: Board, block: Block2, attributes?: Partial<PolygonAttributes>);
    get scaleL(): Geometric2;
    set scaleL(scaleL: Geometric2);
    removeFromBoard(): void;
    update(): void;
    get visible(): boolean;
    set visible(visible: boolean);
}

interface ForceLawViewOptions$1 {
    visible: boolean;
    name: string;
}
declare class JsxConstraint implements JsxWidget {
    private readonly forceLaw;
    scaleL: Geometric2;
    scaleF: Geometric2;
    readonly visible: boolean;
    private readonly arrows;
    constructor(board: Board, forceLaw: SurfaceConstraint2, options?: Partial<ForceLawViewOptions$1>);
    removeFromBoard(): void;
    update(): void;
}

declare class JsxDisc implements JsxWidget {
    private readonly board;
    private readonly disc;
    readonly centerPoint: Point;
    private readonly radiusPoint;
    private readonly circle;
    private readonly $scaleL;
    private $visible;
    constructor(board: Board, disc: Disc2, attributes?: Partial<CircleAttributes>);
    get scaleL(): Geometric2;
    set scaleL(scaleL: Geometric2);
    removeFromBoard(): void;
    update(): void;
    get visible(): boolean;
    set visible(visible: boolean);
}

interface JsxForceAttributes extends ArrowAttributes {
}
declare class JsxForce implements JsxWidget {
    private readonly board;
    readonly force: Force2;
    private readonly arrow;
    private readonly $scaleF;
    private readonly $scaleL;
    private $visible;
    constructor(board: Board, force: Force2, attributes?: Partial<JsxForceAttributes>);
    removeFromBoard(): void;
    update(): void;
    get scaleF(): Geometric2;
    set scaleF(scaleF: Geometric2);
    get scaleL(): Geometric2;
    set scaleL(scaleL: Geometric2);
    get visible(): boolean;
    set visible(visible: boolean);
}

/**
 * A rendering of a davinci-newton Particle2 on a JsxGraph board.
 */
declare class JsxParticle implements JsxWidget {
    private readonly board;
    private readonly particle;
    private readonly point;
    private $visible;
    /**
     * @param board The JsxGraph board that this JsxParticle will be rendered to.
     * @param particle The model for this rendering.
     * @param attributes The optional attributes for the creation of the underlying Point.
     */
    constructor(board: Board, particle: Particle2, attributes?: Partial<PointAttributes>);
    removeFromBoard(): void;
    update(): void;
    get visible(): boolean;
    set visible(visible: boolean);
}

declare class JsxRod implements JsxWidget {
    private readonly board;
    private readonly rod;
    private negPoint;
    private posPoint;
    private readonly segment;
    private readonly $scaleL;
    private $visible;
    constructor(board: Board, rod: Rod2, attributes?: Partial<SegmentAttributes>);
    get scaleL(): Geometric2;
    set scaleL(scaleL: Geometric2);
    removeFromBoard(): void;
    update(): void;
    get visible(): boolean;
    set visible(visible: boolean);
}

interface SpringAttributes {
    radius: number;
    rings: number;
}
declare class JsxSpring implements JsxWidget {
    private readonly board;
    private readonly spring;
    private readonly p1;
    private readonly p2;
    private readonly points;
    private readonly segments;
    private $visible;
    private readonly $scaleL;
    constructor(board: Board, spring: Spring<Geometric2>, config: SpringAttributes);
    get scaleL(): Geometric2;
    set scaleL(scaleL: Geometric2);
    removeFromBoard(): void;
    update(): void;
    get visible(): boolean;
    set visible(visible: boolean);
    get end1(): Point;
}

/**
 * A wrapper around a JSXGraph board providing methods for adding objects to
 * a Free Body Diagram.
 */
declare class JsxDiagram {
    readonly board: Board;
    private readonly views;
    /**
     * @param elementId HTML identifier (id) of the DIV element in which the diagram is rendered.
     * @param attributes an object that sets some of the diagram properties.
     */
    constructor(elementId: string, attributes?: Partial<BoardAttributes>);
    addAngle(attributes?: Partial<AngleAttributes>): JsxAngle;
    addArrow(attributes?: Partial<JsxArrowAttributes>): JsxArrow;
    addDisc(disc: Disc2, attributes?: Partial<CircleAttributes>): JsxDisc;
    addForce(force: Force2, attributes?: Partial<JsxForceAttributes>): JsxForce;
    addParticle(particle: Particle2, attributes?: Partial<PointAttributes>): JsxParticle;
    addRod(rod: Rod2, attributes?: Partial<SegmentAttributes>): JsxRod;
    addSpring(spring: Spring<Geometric2$1>, attributes: SpringAttributes): JsxSpring;
    removeAllFromBoard(): void;
    update(): void;
}

interface ForceLawViewOptions {
    visible: boolean[];
    names: string[];
    colors: string[];
}
declare class JsxForceLaw {
    private readonly forceLaw;
    scaleL: Geometric2;
    scaleF: Geometric2;
    readonly visible: boolean[];
    private readonly arrows;
    constructor(board: Board, forceLaw: ForceLaw<Geometric2>, options?: Partial<ForceLawViewOptions>);
    update(): void;
}

declare class JsxPolygon implements JsxWidget {
    private readonly board;
    private readonly body;
    private jxgPoints;
    private jxgX;
    private readonly polygon;
    private readonly $scaleL;
    private $visible;
    constructor(board: Board, body: Polygon2, attributes?: Partial<PolygonAttributes>);
    get scaleL(): Geometric2;
    set scaleL(scaleL: Geometric2);
    removeFromBoard(): void;
    update(): void;
    get visible(): boolean;
    set visible(visible: boolean);
}

declare class JsxTapeMeasure implements JsxWidget {
    private readonly beginFn;
    private readonly endFn;
    b: Point;
    e: Point;
    scaleL: Geometric2;
    standoff: number;
    private readonly tapemeasure;
    private $visible;
    constructor(board: Board, beginFn: () => Geometric2, endFn: () => Geometric2);
    removeFromBoard(): void;
    update(): void;
    get visible(): boolean;
    set visible(visible: boolean);
}

export { JsxAngle, JsxArrow, JsxArrowAttributes, JsxBlock, JsxConstraint, JsxDiagram, JsxDisc, JsxForce, JsxForceAttributes, JsxForceLaw, JsxParticle, JsxPolygon, JsxRod, JsxSpring, JsxTapeMeasure, JsxWidget };
